@@ -1,7 +1,12 @@
+from __future__ import annotations
+from typing import TYPE_CHECKING
 from ormlambda.sql.clause_info import AggregateFunctionBase
 from ormlambda.sql.types import ColumnType, AliasType
 from ormlambda.sql.clause_info.clause_info_context import ClauseContextType
 
+if TYPE_CHECKING:
+    from ormlambda.dialects import Dialect
+
 
 class ST_AsText[T, TProp](AggregateFunctionBase[None]):
     """
@@ -20,16 +25,12 @@ class ST_AsText[T, TProp](AggregateFunctionBase[None]):
         alias_table: AliasType[ColumnType[TProp]] = None,
         alias_clause: AliasType[ColumnType[TProp]] = None,
         context: ClauseContextType = None,
+        *,
+        dialect: Dialect,
     ) -> None:
         default_alias_clause = self.create_alias_from(point) if not alias_clause else alias_clause
 
-        super().__init__(
-            table=point.table,
-            column=point,
-            alias_table=alias_table,
-            alias_clause=default_alias_clause,
-            context=context,
-        )
+        super().__init__(table=point.table, column=point, alias_table=alias_table, alias_clause=default_alias_clause, context=context, dialect=dialect)
 
     @staticmethod
     def create_alias_from(element: ColumnType[TProp]) -> str:
