@@ -1,13 +1,14 @@
 from __future__ import annotations
-from typing import Callable, TYPE_CHECKING, Optional, Any, Type, cast, overload
+from typing import Callable, TYPE_CHECKING, Optional, Any, Type, cast, overload, ClassVar
 
 from ormlambda.common.interfaces.IQueryCommand import IQuery
-
+from ormlambda.sql.elements import Element
 
 if TYPE_CHECKING:
     from ormlambda.sql.comparer import Comparer
     from ormlambda import Table
     from ormlambda.sql.clause_info.clause_info_context import ClauseContextType
+    from ormlambda.dialects import Dialect
 
 
 class ForeignKeyContext(set["ForeignKey"]):
@@ -32,49 +33,51 @@ class ForeignKeyContext(set["ForeignKey"]):
         return super().add(element)
 
 
-class ForeignKey[TLeft: Table, TRight: Table](IQuery):
-    stored_calls: ForeignKeyContext = ForeignKeyContext()
+class ForeignKey[TLeft: Table, TRight: Table](Element, IQuery):
+    __visit_name__ = "foreign_key"
+
+    stored_calls: ClassVar[ForeignKeyContext] = ForeignKeyContext()
 
     @overload
-    def __new__[LProp, RProp](self, comparer: Comparer[LProp, RProp], clause_name: str) -> None: ...
+    def __new__(self, comparer: Comparer, clause_name: str) -> None: ...
     @overload
-    def __new__[LProp, TRight, RProp](cls, tright: Type[TRight], relationship: Callable[[TLeft, TRight], Any | Comparer[TLeft, LProp, TRight, RProp]], keep_alive: bool = ...) -> TRight: ...
-
-    def __new__[LProp, TRight, RProp](
+    def __new__[TRight](
         cls,
-        tright: Optional[TRight] = None,
-        relationship: Optional[Callable[[TLeft, TRight], Any | Comparer[TLeft, LProp, TRight, RProp]]] = None,
-        *,
-        comparer: Optional[Comparer] = None,
-        clause_name: Optional[str] = None,
-        keep_alive: bool = False,
-    ) -> TRight:
+        tright: Type[TRight],
+        relationship: Callable[[TLeft, TRight], Any | Comparer],
+        keep_alive: bool = ...,
+        dialect: Dialect = ...,
+    ) -> TRight: ...
+
+    def __new__[TRight](cls, *args, **kwargs) -> TRight:
         return super().__new__(cls)
 
-    def __init__[LProp, RProp](
+    def __init__(
         self,
         tright: Optional[TRight] = None,
-        relationship: Optional[Callable[[TLeft, TRight], Any | Comparer[TLeft, LProp, TRight, RProp]]] = None,
+        relationship: Optional[Callable[[TLeft, TRight], Any | Comparer]] = None,
         *,
         comparer: Optional[Comparer] = None,
         clause_name: Optional[str] = None,
         keep_alive: bool = False,
+        **kwargs: Any,
     ) -> None:
+        self.kwargs = kwargs
         self._keep_alive = keep_alive
         if comparer is not None and clause_name is not None:
-            self.__init__with_comparer(comparer, clause_name)
+            self.__init__with_comparer(comparer, clause_name, **kwargs)
         else:
             self.__init_with_callable(tright, relationship)
 
-    def __init__with_comparer[LProp, RProp](self, comparer: Comparer[LProp, RProp], clause_name: str) -> None:
+    def __init__with_comparer(self, comparer: Comparer, clause_name: str, **kwargs) -> None:
         self._relationship = None
-        self._tleft: TLeft = comparer.left_condition.table
-        self._tright: TRight = comparer.right_condition.table
+        self._tleft: TLeft = comparer.left_condition(**kwargs).table
+        self._tright: TRight = comparer.right_condition(**kwargs).table
         self._clause_name: str = clause_name
-        self._comparer: Comparer[LProp, RProp] = comparer
+        self._comparer: Comparer = comparer
 
-    def __init_with_callable[LProp, RProp](self, tright: Optional[TRight], relationship: Optional[Callable[[TLeft, TRight], Comparer[LProp, RProp]]]) -> None:
-        self._relationship: Callable[[TLeft, TRight], Comparer[LProp, RProp]] = relationship
+    def __init_with_callable(self, tright: Optional[TRight], relationship: Optional[Callable[[TLeft, TRight], Comparer]]) -> None:
+        self._relationship: Callable[[TLeft, TRight], Comparer] = relationship
         self._tleft: TLeft = None
         self._tright: TRight = tright
         self._clause_name: str = None
@@ -113,29 +116,29 @@ class ForeignKey[TLeft: Table, TRight: Table](IQuery):
     def clause_name(self) -> str:
         return self._clause_name
 
-    @property
-    def query(self) -> str:
-        compare = self.resolved_function()
-        rcon = alias if (alias := compare.right_condition.alias_table) else compare.right_condition.table.__table_name__
-        return f"FOREIGN KEY ({self._tleft.__table_name__}) REFERENCES {rcon}({compare.right_condition._column.column_name})"
+    def query(self, dialect: Dialect, **kwargs) -> str:
+        compare = self.resolved_function(dialect)
+        left_col = compare.left_condition(dialect).column
+        rcon = alias if (alias := compare.right_condition(dialect).alias_table) else compare.right_condition(dialect).table.__table_name__
+        return f"FOREIGN KEY ({left_col}) REFERENCES {rcon}({compare.right_condition(dialect).column})"
 
-    @property
-    def alias(self) -> str:
-        self._comparer = self.resolved_function()
-        lcol = self._comparer.left_condition._column.column_name
-        rcol = self._comparer.right_condition._column.column_name
+    def get_alias(self, dialect: Dialect) -> str:
+        self._comparer = self.resolved_function(dialect)
+        self._comparer._dialect = dialect
+        lcol = self._comparer.left_condition(dialect)._column.column_name
+        rcol = self._comparer.right_condition(dialect)._column.column_name
         return f"{self.tleft.__table_name__}_{lcol}_{rcol}"
 
     @classmethod
-    def create_query(cls, orig_table: Table) -> list[str]:
+    def create_query(cls, orig_table: Table, dialect: Dialect) -> list[str]:
         clauses: list[str] = []
 
-        for attr in vars(orig_table):
+        for attr in orig_table.__dict__.values():
             if isinstance(attr, ForeignKey):
-                clauses.append(attr.query)
+                clauses.append(attr.query(dialect))
         return clauses
 
-    def resolved_function[LProp: Any, RProp: Any](self, context: ClauseContextType = None) -> Comparer[LProp, RProp]:
+    def resolved_function[LProp: Any, RProp: Any](self, dialect: Dialect, context: Optional[ClauseContextType] = None) -> Comparer:
         """ """
         if self._comparer is not None:
             return self._comparer
@@ -144,4 +147,5 @@ class ForeignKey[TLeft: Table, TRight: Table](IQuery):
         right = self._tright
         comparer = self._relationship(left, right)
         comparer.set_context(context)
+        comparer._dialect = dialect
         return comparer
